#include "TNR_SHAPE.hxx"
#include "TNR_VECTOR.hxx"
#include "TNR_CURVE.hxx"
#include "TNR_CONFIG.hxx"
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

///////////////////////////
//TNR_LINE
///////////////////////////
TNR_LINE::TNR_LINE(TNR_POINT p0,TNR_POINT p1)
{
  _p[0] = p0;
  _p[1] = p1;
}

TNR_POINT TNR_LINE::getPosition(double u)
{
 

  return  u*(_p[1] - _p[0]) + _p[0];
}

double TNR_LINE::getLength()
{
  return distanceBetween(_p[0],_p[1]);
}
  

///////////////////////////
//TNR_QUADRIC_CURVE
///////////////////////////
TNR_QUADRIC_CURVE::TNR_QUADRIC_CURVE(TNR_POINT p0,TNR_POINT p1,TNR_POINT p2)
{
  _p[0] = p0;
  _p[1] = p1;
  _p[2] = p2;
}

TNR_POINT TNR_QUADRIC_CURVE::getPosition(double pu)
{
  double u  = ( pu * 2.0 ) - 1.0;

  double k0 = 0.5 * u *  (u - 1.0); 
  double k1 =(1.0 -u ) * (1.0 + u);
  double k2 = 0.5 * u *  (u + 1.0);

  TNR_POINT pos = k0*_p[0] + k1*_p[1] + k2*_p[2];
  return pos;
}

double TNR_QUADRIC_CURVE::getLength()
{
  TNR_POINT p0 = this->getPosition(0.0);
  TNR_POINT p1;
  int div = 10;
  double incre= 1.0/div;
  double paraPos = incre;
  double  dist=0.0;

  for(int i = 1; i < div+1;i++){
     p1 = this->getPosition(paraPos);
     dist= dist + distanceBetween(p0,p1);
	 p0 = p1;
	 paraPos= paraPos+incre;
  }

 return dist;
}

///////////////////////////
//TNR_CUBIC_CURVE
///////////////////////////
TNR_CUBIC_CURVE::TNR_CUBIC_CURVE(TNR_POINT p0,TNR_POINT p1,TNR_POINT p2,TNR_POINT p3)
{
  _p[0] = p0;
  _p[1] = p1;
  _p[2] = p2;
  _p[3] = p3;
}

TNR_POINT TNR_CUBIC_CURVE::getPosition(double u)
{

  double k0 =  0.5 * ( 1.0 - u ) * ( 1.0 - 3.0 * u ) * (2.0 - 3.0 * u );
  double k1 =  4.5 * u * ( 1.0 - u ) * ( 2.0 - 3.0 * u );
  double k2 = -4.5 * u * ( 1.0 - u ) * ( 1.0 - 3.0 * u );
  double k3 =  0.5 * u *  ( 1.0 - 3.0 * u ) * (2.0 - 3.0 * u );

  TNR_POINT pos = k0 * _p[0] + k1 * _p[1] + k2 * _p[2] + k2 * _p[3];
  return pos;
}

double TNR_CUBIC_CURVE::getLength()
{
  TNR_POINT p0 = this->getPosition(0.0);
  TNR_POINT p1;
  int div = 10;
  double incre= 1.0/div;
  double paraPos = incre;
  double  dist=0.0;

  for(int i = 1; i < div+1;i++){
     p1 = this->getPosition(paraPos);
     dist= dist + distanceBetween(p0,p1);
	 p0 = p1;
	 paraPos= paraPos+incre;
  }

 return dist;
}


  
///////////////////////////
// TNR_CIRCLE
///////////////////////////
TNR_CIRCLE::TNR_CIRCLE(TNR_POINT center,double radius)
{
  _center = center;
  _r = radius;
}

TNR_POINT TNR_CIRCLE::getPosition(double u)
{
  TNR_POINT pos( _r * cos(u * 2 * M_PI), _r * sin(u * 2 * M_PI) ,0.0);
  return _center+pos;
}

double TNR_CIRCLE::getLength()
{
 return 2 * M_PI * _r;
}
 
///////////////////////////
// TNR_ELLIPSE
///////////////////////////
TNR_ELLIPSE::TNR_ELLIPSE(TNR_POINT center,double majorAxisRadius,double minorAxisRadius)
{
  _center = center;
  _a = majorAxisRadius;
  _b = minorAxisRadius;
}


TNR_POINT TNR_ELLIPSE::getPosition(double u)
{
  TNR_POINT pos(_a * cos( u * 2 * M_PI ), _b * sin(u * 2 * M_PI ),0.0);
  return _center+pos;
}

double TNR_ELLIPSE::getLength()
{
  return M_PI * sqrt( 2 * ( _a *_a + _b * _b ) ) - ( _a - _b ) * ( _a - _b ) / 2.2;
}
 
