#ifndef __TNR_INI_HXX
#define __TNR_INI_HXX

#include <string>
#include <map>
#include <vector>
using namespace std;

string trimSpace(const string& s);
string trimDobleQuotes(const string& s);



class TNR_INI;

const int M_BUFFER_SIZE = 256;
const int BUFFER_SIZE = 1024;

class TNR_KEY{
	string _key;
	string _value;
public:
	TNR_KEY(){}
	TNR_KEY(string& key,string& value){_key = key;this->setValue(value);}
	TNR_KEY(string& key,double value){ _key = key;this->setValue(value);}
	TNR_KEY(string& key,int value){ _key = key;this->setValue(value);}

	~TNR_KEY(){}
public:
    void   setKey(string& key){_key = key;}
    void  setValue(string& value){_value=value;}
    void   setValue(double value){
		char buf[256];sprintf(buf,"%f",value);
		_value= buf;}
	void   setValue(int value){
		char buf[256];sprintf(buf,"%d",value);
		_value= buf;}
public:
	string getKey(){return _key;}
	string getStringValue(){return _value;}
	int    getIntValue(){return atoi(_value.c_str());}
	double getDoubleValue(){return atof(_value.c_str());}
};





class TNR_COMAMND{
public:
  string    _commandName;
  vector<string> _keys;
  vector<string> _values;
public:
	TNR_COMAMND(string& line){this->setCommandLine(line);}
  void setCommandLine(string& line);
  bool   isDefinedKey(const string& key)
  {
	for(int i = 0; i < _keys.size();i++){
      if(_keys[i] == key)
		return true;
	  }
    return false;

  }

  string getCommandName(){return _commandName;}
  string getKey(int index){return _keys[index];}
  string getValue(int index){return _values[index];}
  string getStringValue(const string& key,string default);
  int    getIntValue(const string& key,int default);
  double   getDoubleValue(const string& key,double default);
  void print(FILE*fp); 
};


class TNR_SECTION{
  //subset of  init file format
  string           _name;//section name
  vector<TNR_KEY*> _keys;

  //data of secton
  vector<string>     _rowData;
  TNR_INI*           _owner;

public:
	TNR_SECTION(){ this->initilize();}
	TNR_SECTION(string& name,TNR_INI* owner){ this->initilize();_name = name;_owner = owner;}
	TNR_SECTION(const char* name,TNR_INI* owner){ this->initilize();_name = name;_owner = owner;}
	~TNR_SECTION(){}
public:
	void  initilize(){_owner = NULL;}
	void  setOwner(TNR_INI* owner){_owner = owner;}
	TNR_INI*  getOwner(){return _owner;}
	string getName(){return _name;}
	int getNumKey(){return _keys.size();}
	TNR_KEY* getKey(int i){return _keys[i];}

	bool  isDefinedKey(const string& key)
	{
		for(int i = 0; i < _keys.size();i++){
          if(_keys[i]->getKey() == key)
			return true;
		}
        return false;
	}
	string getStringValue(const string& key,string default)
	{
		for(int i = 0; i < _keys.size();i++){
          if(_keys[i]->getKey() == key)
			return _keys[i]->getStringValue();
		}
        return default;
	}



	int getIntValue(const string& key,int default)
	{   for(int i = 0; i < _keys.size();i++){
          if(_keys[i]->getKey() == key)
			return _keys[i]->getIntValue();
		}
        return  default;
	}
	
	double getDoubleValue(const string& key,double default)
	{   for(int i = 0; i < _keys.size();i++){
          if(_keys[i]->getKey() == key)
			return _keys[i]->getDoubleValue();
		}
        return default;
	}
	void setName(string& sectionName){_name = sectionName;}

	void  addKeyValue(TNR_KEY* keyValue){_keys.push_back(keyValue);}
	void  addKeyStringValue(string key, string value)
	{
       for(int i = 0; i < _keys.size();i++){
	     if(_keys[i]->getKey() == key){
	       _keys[i]->setValue(value);
		   return;
		 }
	   }
	   TNR_KEY* keyValue = new TNR_KEY(key,value);_keys.push_back(keyValue);
    }
	void  addKeyIntValue(string key, int value)
	{
       for(int i = 0; i < _keys.size();i++){
	     if(_keys[i]->getKey() == key){
	       _keys[i]->setValue(value);
		   return;
		 }
	   }
	   TNR_KEY* keyValue = new TNR_KEY(key,value);_keys.push_back(keyValue);

    }
	void  addKeyDoubleValue(string key,double value)
	{
       for(int i = 0; i < _keys.size();i++){
	     if(_keys[i]->getKey() == key){
	       _keys[i]->setValue(value);
		   return;
		 }
	   }
	   TNR_KEY* keyValue = new TNR_KEY(key,value);_keys.push_back(keyValue);

    }
	    
	void  addData(char* data){_rowData.push_back(data);}
	void  addData(string& data){_rowData.push_back(data);}
	void  setData(int i,string& data){_rowData[i] = data;}
	int  getNumData(){return _rowData.size();}
	string  getData(int index){return _rowData[index];}
	void  copy(TNR_SECTION& s);
};

class TNR_INI{
	vector<TNR_SECTION*> _sections;
	TNR_SECTION* _thisSection;	
public:
	TNR_INI();
	~TNR_INI();
public:
	bool load(const char* iniFileName);
	bool save(const char* iniFileName);
public:
	bool AddData(string& key,string& value);
	bool AddSection(string& sectionName);
	bool AnalysisOneString(string& buff);
	bool AnalysisString(string& buffer);
public:
	int getNumSection(){return _sections.size();}
	TNR_SECTION*  getSection(int i){return _sections[i];}
public:
	void addSection(TNR_SECTION* sec){_sections.push_back(sec);}
	void deleteSectionByKey(string searchKey,string searchValue);
	void getSection(const string& sectionName,vector<TNR_SECTION*>& returnSection);
	bool getSection(const string& searchSectionName,const string searchKey,string searchValue,vector<TNR_SECTION*>& returnSections);
	bool getSection(const string searchKey,string searchValue,vector<TNR_SECTION*>& returnSections);
};



#endif





 
