#include "TNR_QUAD_MESH.hxx"
#include "TNR_COLLECTION.hxx"
#include "TNR_SURFACE.hxx"
#include <stdlib.h>
#include <stdio.h>
//#include <windows.h>
#include <string>
#include <string.h>
#include <assert.h>
#include <math.h>

bool TNR_QUAD_MESH
::isConvex(int i)
{
 //is convex
  
  TNR_POINT p0 = _points[_quads[i][0]];
  TNR_POINT p1 = _points[_quads[i][1]];
  TNR_POINT p2 = _points[_quads[i][2]];
  TNR_POINT p3 = _points[_quads[i][3]];

  TNR_VECTOR  a0 = crossProduct( ( p1-p0 ),( p3-p0 ) );
  TNR_VECTOR  a1 = crossProduct( ( p2-p1 ),( p0-p1 ) );
  TNR_VECTOR  a2 = crossProduct( ( p3-p2 ),( p1-p2 ) );
  TNR_VECTOR  a3 = crossProduct( ( p0-p3 ),( p2-p3 ) );

  double result0 = dotProduct(a0,a1);
  double result1 = dotProduct(a0,a2);
  double result2 = dotProduct(a0,a3);

  if(result0 < 0.0)
	  return false;

  if(result1 < 0.0)
	  return false;

  if(result2 < 0.0)
	  return false;

  return true;

}


void TNR_QUAD_MESH
::createMesh(TNR_SECTION* surface,
		  TNR_COMAMND* commands)
{
  //create surface for meshing of shape
  string surfaceType = surface->getStringValue("surface_type","");
  TNR_POINT ctlPoints[12];
  int numPoins = surface->getNumData();
  for(int i = 0; i < numPoins; i++){
    string data = surface->getData(i);
    sscanf(data.c_str(),"%lf %lf %lf", &(ctlPoints[i][0]),&(ctlPoints[i][1]),&(ctlPoints[i][2]));
  }

  TNR_SURFACE* s;
  if(surfaceType == "bilinearSurface"){
     s = (TNR_SURFACE*) new TNR_BILINEAR_SURFACE(ctlPoints);
  }
  if(surfaceType == "quadricSurface"){
     s = (TNR_SURFACE*) new TNR_QUADRIC_SURFACE(ctlPoints);
  }
  if(surfaceType == "cubicSurface"){
     s = (TNR_SURFACE*) new TNR_CUBIC_SURFACE(ctlPoints);
  }
  if(surfaceType == "circleSurface"){
    double radius =  surface->getDoubleValue("radius",1.0);
    string center =  surface->getStringValue("center","0.0 0.0 0.0");
    TNR_POINT centerPos;
    sscanf(center.c_str(),"%lf %lf %lf",&(centerPos[0]),&(centerPos[1]),&(centerPos[2]));
    s = (TNR_SURFACE*) new TNR_CIRCLE_SURFACE(centerPos,radius);
  }
  if(surfaceType == "ellipseSurface"){
	double majorRadius =  surface->getDoubleValue("major_axis_radius",1.0);
	double minorRadius =  surface->getDoubleValue("minor_axis_radius",1.0);
    string center      =  surface->getStringValue("center","0.0 0.0 0.0");
    TNR_POINT centerPos;
    sscanf(center.c_str(),"%lf %lf %lf",&(centerPos[0]),&(centerPos[1]),&(centerPos[2]));
    s = (TNR_SURFACE*) new TNR_ELLIPSE_SURFACE(centerPos,majorRadius,minorRadius);
  }

  //get mesh parameter
  int  udiv,vdiv;
  if(commands->isDefinedKey("base_size") == true){
    double base = commands->getDoubleValue("base_size",1.0);
	double a = s->getUMaximumSideLength();
	double b = s->getVMaximumSideLength();
	udiv =  int( 0.5 + (s->getUMaximumSideLength() / base));
	vdiv =  int( 0.5 + (s->getVMaximumSideLength() / base));
  }else{
    udiv = commands->getIntValue("u_div",1.0);
    vdiv = commands->getIntValue("v_div",1.0);
  }


  //initilize
  int numMeshPoints = (udiv+1)*(vdiv+1);
  int numMeshQuads = udiv*vdiv;
  this->initilize(numMeshPoints,numMeshQuads);

  int uloop = udiv + 1;
  int vloop = vdiv + 1;

  //set edge points 
  _edgePoints[0].reserve(uloop);
  _edgePoints[1].reserve(vloop);
  _edgePoints[2].reserve(uloop);
  _edgePoints[3].reserve(vloop);

  for(int i = 0; i < uloop;i++){
    int val = i;
    _edgePoints[0].push_back(i);
  }

  for(int i = uloop-1; i < vloop*uloop;i+=uloop){
    _edgePoints[1].push_back(i);
  }

  for(int i = uloop*(vloop-1); i < uloop*vloop;i++){
    int val = i;
    _edgePoints[2].push_back(i);
  }

  for(int i = 0; i < _edgePoints[1].size();i++){
    int val = _edgePoints[1][i] - (uloop-1);
    _edgePoints[3].push_back(val);
  }
  //end of edge points;

  int posCount = 0;
  double uIncre = 1.0 / udiv;
  double vIncre = 1.0 / vdiv;
  double uPos = 0.0;
  double vPos = 0.0;

  //create points
  for(int v = 0; v < vloop; v++){
     uPos = 0.0;	  
     for(int u = 0; u < uloop; u++){
	   this->setPoint(posCount,s->getPosition(uPos,vPos));
       uPos += uIncre;
	   posCount++;
	 }
	 vPos += vIncre;
  }

  //create quad meshes
 
  int iPos = 0;
  int iQuad = 0;
  for(int v = 0; v < vdiv; v++){
    for(int u = 0; u < udiv; u++){
	  TNR_QUAD quad;
	  quad[0] = iPos;
	  quad[1] = iPos+1;
	  quad[2] = quad[0]+udiv+2;
	  quad[3] = quad[2] -1;

	  this->setMesh(iQuad,quad);
	  iPos ++;
	  iQuad++;
	}
	iPos ++;
  }

  //projection
  this->projectionToBoundary(surface);
  // smooth 
  if(surface->getStringValue("smooth","off") != "off" ){
    this->smoothInternalPoints();
  }

  delete  s;

}

void TNR_QUAD_MESH
::projectionToBoundary(TNR_SECTION* surface)
{
  for(int edgeId=0; edgeId< 4;edgeId++){
    char buf[1024];
    sprintf(buf,"edge[%d]",edgeId);
	string edgeName = surface->getStringValue(string(buf),"");
    
	//projection edge
	if(surface->getStringValue(string(buf),"") != ""){
	  TNR_INI* ini = surface->getOwner();
	  vector<TNR_SECTION*> curves;
	  ini->getSection("edge","name",edgeName, curves);
	  if( curves.size() != 1){
		printf("#ERR curve\n");
        return;
	  }
	  if(curves[0]->getStringValue("curve_type","") == "circle"){
		TNR_POINT centerPos;
        string center = curves[0]->getStringValue(string ("center"),"0.0,0.0,0.0");     
		double raduis = curves[0]->getDoubleValue(string("radius"),1.0);

        if(center != ""){
          sscanf(center.c_str(),"%lf,%lf,%lf",&(centerPos[0]),&(centerPos[1]),&(centerPos[2]));
		}

		for(int j = 0; j < _edgePoints[edgeId].size();j++){
		   int nodeId = _edgePoints[edgeId][j];
		   TNR_VECTOR vec = this->getPoint(nodeId) - centerPos;
		   TNR_POINT pos = centerPos + (raduis * vec.normalize());
		  this->setPoint(nodeId,pos);
		}
	  } 
	}
  }
}
void TNR_QUAD_MESH
::smoothInternalPoints()
{
  //ߓ_Ăvf̃e[u
  vector<int>*      countMesh     = new  vector<int>[this->getNumPoints()];
  TNR_POINT org;
  TNR_POINT mv;

  for(int i = 0; i < this->getNumMeshes();i++){
    TNR_QUAD q = this->getQuad(i);
	countMesh[q[0]].push_back(i);
	countMesh[q[1]].push_back(i);
	countMesh[q[2]].push_back(i);
	countMesh[q[3]].push_back(i);
  }

  for(int loop = 0 ; loop < 10;loop++){
    for(int i = 0; i < this->getNumPoints();i++){


	  //̓_
	  TNR_IntArray a;
	  if(countMesh[i].size() == 4){

	    //ߓ_
	    for(int j = 0; j < 4; j++){
		  TNR_QUAD q =   this->getQuad(countMesh[i][j]);
		  a.uniqe_add(q[0]);
		  a.uniqe_add(q[1]);
		  a.uniqe_add(q[2]);
		  a.uniqe_add(q[3]);
	    }

	    //backup orginal node
		org =  this->getPoint(i);
		mv  =  this->getPoint(i);

	    for(int j = 0; j < a.size(); j++){
		  if(a[j] != i){
		    mv+=   this->getPoint(a[j]);
		  }
	    }
		//
	    mv =  mv / a.size();
		this->setPoint(i,mv);

		//if quad is not Convex,node dont move.
	    for(int j = 0; j < 4; j++){
		  if(this->isConvex(countMesh[i][j]) == false){
            this->setPoint(i,org);
		  }
	    }

	  }//end of one node

    }//end of all node smooth 

  }//end of smooth loop 
}

