
#include <stdlib.h>
#include <stdio.h>
#include <string>
#include "TNR_SOLID.hxx"
#include "TNR_CURVE.hxx"

///////////////////////
//TNR_CUBE8
///////////////////////
TNR_CUBE8::TNR_CUBE8(TNR_POINT  p[8])
{
  for(int i = 0; i < 8; i++){
    _p[i] = p[i];
  }
}



TNR_POINT TNR_CUBE8::getPosition(double pu,double pv,double pt)
{
 double u   = ( pu * 2.0 ) - 1.0;
 double v   = ( pv * 2.0 ) - 1.0;
 double t   = ( pt * 2.0 ) - 1.0;

 double N0 = ( ( 1.0 + u ) * ( 1.0 - v ) * (1.0 - t) ) / 8.0;
 double N1 = ( ( 1.0 + u ) * ( 1.0 + v ) * (1.0 - t) ) / 8.0;
 double N2 = ( ( 1.0 - u ) * ( 1.0 + v ) * (1.0 - t) ) / 8.0;
 double N3 = ( ( 1.0 - u ) * ( 1.0 - v ) * (1.0 - t) ) / 8.0;
 double N4 = ( ( 1.0 + u ) * ( 1.0 - v ) * (1.0 + t) ) / 8.0;
 double N5 = ( ( 1.0 + u ) * ( 1.0 + v ) * (1.0 + t) ) / 8.0;
 double N6 = ( ( 1.0 - u ) * ( 1.0 + v ) * (1.0 + t) ) / 8.0;
 double N7 = ( ( 1.0 - u ) * ( 1.0 - v ) * (1.0 + t) ) / 8.0;

 TNR_POINT p = N0*_p[0] + N1*_p[1] + N2*_p[2] + N3*_p[3] + N4*_p[4] + N5*_p[5] + N6*_p[6] + N7*_p[7];
 return p;
}

double  TNR_CUBE8::getUMaximumSideLength()
{
   return distanceBetween(_p[3],_p[0]);
}

double  TNR_CUBE8::getVMaximumSideLength()
{
  return distanceBetween(_p[2],_p[3]);
}

double  TNR_CUBE8::getTMaximumSideLength()
{
  return distanceBetween(_p[3],_p[7]);
}

///////////////////////
//TNR_CUBE20
///////////////////////
TNR_CUBE20::TNR_CUBE20(TNR_POINT  p[20])
{
  for(int i = 0; i < 20; i++){
    _p[i] = p[i];
  }
}


TNR_POINT TNR_CUBE20::getPosition(double pu,double pv,double pt)
{
 double u  = ( pu * 2.0 ) - 1.0;
 double v  = ( pv * 2.0 ) - 1.0;
 double t  = ( pt * 2.0 ) - 1.0;

 double N0 = 0.125 *  ( 1.0 - u ) * ( 1.0 - v ) * (1.0 - t) * (-2.0 - u - v - t);
 double N1 = 0.125 *  ( 1.0 + u ) * ( 1.0 - v ) * (1.0 - t) * (-2.0 + u - v - t);
 double N2 = 0.125 *  ( 1.0 + u ) * ( 1.0 + v ) * (1.0 - t) * (-2.0 + u + v - t);
 double N3 = 0.125 *  ( 1.0 - u ) * ( 1.0 + v ) * (1.0 - t) * (-2.0 - u + v - t);
 double N4 = 0.125 *  ( 1.0 - u ) * ( 1.0 - v ) * (1.0 + t) * (-2.0 - u - v + t);
 double N5 = 0.125 *  ( 1.0 + u ) * ( 1.0 - v ) * (1.0 + t) * (-2.0 + u - v + t);
 double N6 = 0.125 *  ( 1.0 + u ) * ( 1.0 + v ) * (1.0 + t) * (-2.0 + u + v + t);
 double N7 = 0.125 *  ( 1.0 - u ) * ( 1.0 + v ) * (1.0 + t) * (-2.0 - u + v + t);
 double N8 = 0.25  *  ( 1.0 - u*u ) * ( 1.0 - v ) * (1.0 - t);
 double N9 = 0.25  *  ( 1.0 + u ) * ( 1.0 - v*v ) * (1.0 - t);
 double N10 = 0.25  *  ( 1.0 - u*u ) * ( 1.0 + v ) * (1.0 - t);
 double N11 = 0.25  *  ( 1.0 - u ) * ( 1.0 - v*v ) * (1.0 - t);
 double N12 = 0.25  *  ( 1.0 - u ) * ( 1.0 - v) * (1.0 - t*t);
 double N13 = 0.25  *  ( 1.0 + u ) * ( 1.0 - v) * (1.0 - t*t);
 double N14 = 0.25  *  ( 1.0 + u ) * ( 1.0 + v) * (1.0 - t*t);
 double N15 = 0.25  *  ( 1.0 - u ) * ( 1.0 + v) * (1.0 - t*t);
 double N16 = 0.25  *  ( 1.0 - u*u ) * ( 1.0 - v ) * (1.0 + t);
 double N17 = 0.25  *  ( 1.0 + u ) * ( 1.0 - v*v ) * (1.0 + t);
 double N18 = 0.25  *  ( 1.0 - u*u ) * ( 1.0 + v ) * (1.0 + t);
 double N19 = 0.25  *  ( 1.0 - u ) * ( 1.0 - v*v ) * (1.0 + t);

 TNR_POINT p = N0*_p[0] + N1*_p[1] + N2*_p[2] + N3*_p[3] + N4*_p[4] + N5*_p[5] + N6*_p[6] + N7*_p[7] + N8*_p[8] + N9*_p[9]
               + N10*_p[10]+ N11*_p[11]+ N12*_p[12]+ N13*_p[13]+ N14*_p[14]+ N15*_p[15]+ N16*_p[16] + N17*_p[17] + N18*_p[18] + N19*_p[19];

 return p;
}

double  TNR_CUBE20::getUMaximumSideLength()
{
  TNR_QUADRIC_CURVE curve1(_p[3],_p[11],_p[0]);
  TNR_QUADRIC_CURVE curve2(_p[2],_p[9],_p[1]);
  TNR_QUADRIC_CURVE curve3(_p[6],_p[17],_p[5]);
  TNR_QUADRIC_CURVE curve4(_p[7],_p[19],_p[4]);
  double value1 = curve1.getLength();
  double value2 = curve2.getLength();
  double value3 = curve3.getLength();
  double value4 = curve4.getLength();
  double wk1 = (value1 < value2) ? value2 : value1;
  double wk2 = (value3 < value4) ? value4 : value3;
  return (wk1 <wk2) ? wk2 : wk1;
}

double  TNR_CUBE20::getVMaximumSideLength()
{
  TNR_QUADRIC_CURVE curve1(_p[7],_p[18],_p[6]);
  TNR_QUADRIC_CURVE curve2(_p[3],_p[10],_p[2]);
  TNR_QUADRIC_CURVE curve3(_p[4],_p[16],_p[5]);
  TNR_QUADRIC_CURVE curve4(_p[0],_p[8],_p[1]);
  double value1 = curve1.getLength();
  double value2 = curve2.getLength();
  double value3 = curve3.getLength();
  double value4 = curve4.getLength();
  double wk1 = (value1 < value2) ? value2 : value1;
  double wk2 = (value3 < value4) ? value4 : value3;
  return (wk1 <wk2) ? wk2 : wk1;
}

double  TNR_CUBE20::getTMaximumSideLength()
{
  TNR_QUADRIC_CURVE curve1(_p[7],_p[15],_p[3]);
  TNR_QUADRIC_CURVE curve2(_p[4],_p[12],_p[0]);
  TNR_QUADRIC_CURVE curve3(_p[5],_p[13],_p[1]);
  TNR_QUADRIC_CURVE curve4(_p[6],_p[14],_p[2]);
  double value1 = curve1.getLength();
  double value2 = curve2.getLength();
  double value3 = curve3.getLength();
  double value4 = curve4.getLength();
  double wk1 = (value1 < value2) ? value2 : value1;
  double wk2 = (value3 < value4) ? value4 : value3;
  return (wk1 <wk2) ? wk2 : wk1;
}