#ifndef __TNR_VECTOR_HXX
#define __TNR_VECTOR_HXX

//  depend compiler
#include "TNR_CONFIG.hxx"

/////////////////////////////                       
//     TNR_VECTOR  class   //                       
/////////////////////////////  
class TNR_VECTOR { 
public:
   TNR_VECTOR();
   ~TNR_VECTOR();
   TNR_VECTOR(const double x, const double y, const double z);
   TNR_VECTOR& operator= (const TNR_VECTOR& p);
   TNR_VECTOR& operator+= (const TNR_VECTOR& p);
   TNR_VECTOR& operator-= (const TNR_VECTOR& p);
   TNR_VECTOR& zero();
   void set(double x,double y,double z){_v[0] =x;_v[1] =y;_v[2] =z;}
   inline double x() const { return _v[0]; }
   inline double y() const { return _v[1]; }
   inline double z() const { return _v[2]; }
   double& operator[](int i);
   double& operator()(int i); 
   void upper(TNR_VECTOR& p);
   void lower(TNR_VECTOR& p);
   double norm() const;
   TNR_VECTOR& normalize();
   void move(const TNR_VECTOR& center, TNR_VECTOR& vector);
   void scale(const TNR_VECTOR& center, TNR_VECTOR& factor);
   TNR_VECTOR rotateByXAxis (const TNR_VECTOR& center, double angle);
   TNR_VECTOR rotateByYAxis (const TNR_VECTOR& center, double angle);
   TNR_VECTOR rotateByZAxis (const TNR_VECTOR& center, double angle);
   friend TNR_VECTOR operator* (const double a, const TNR_VECTOR& p);
   friend TNR_VECTOR operator* (const TNR_VECTOR& p0,const TNR_VECTOR& p1);
   friend TNR_VECTOR operator/ (const TNR_VECTOR& p,const double a);
   friend TNR_VECTOR operator/ (const TNR_VECTOR& p0,const TNR_VECTOR& p1);
   friend TNR_VECTOR operator+ (const TNR_VECTOR& p1, const TNR_VECTOR& p2);
   friend TNR_VECTOR operator+ (const TNR_VECTOR& p1, const double m);
   friend TNR_VECTOR operator- (const TNR_VECTOR& p1, const TNR_VECTOR& p2);
   friend TNR_VECTOR operator- (const TNR_VECTOR& p1, const double m);
   friend int operator== (const TNR_VECTOR& a, const TNR_VECTOR& b);
   friend int operator!= (const TNR_VECTOR& a, const TNR_VECTOR& b);
   friend double dotProduct(const TNR_VECTOR& a, const TNR_VECTOR& b);
   friend TNR_VECTOR crossProduct(const TNR_VECTOR& a, const TNR_VECTOR& b);
   friend double distanceBetween(const TNR_VECTOR& a, const TNR_VECTOR& b);
   friend double angleBetween(const TNR_VECTOR& vec01, const TNR_VECTOR& vec02);
private:
   double _v[3]; 
};

typedef TNR_VECTOR  TNR_POINT;

#endif
