#include <stdlib.h>
#include <stdio.h>

#include "TNR_INI.hxx"
#include "TNR_VECTOR.hxx"
#include "TNR_MESH.hxx"
#include "TNR_QUAD_MESH.hxx"
#include "TNR_HEX_MESH.hxx"
#include "TNR_MESH_MODEL.hxx"
#include "TNR_FILE_VTU.hxx"
#include "TNR_COMMON.hxx"
#include "TNR_SHAPE.hxx"
#include "TNR_TRANSFORM.hxx"
#include "TNR_CURVE.hxx"
#include <math.h>


int main(int argc,char* argv[])
{

  if(argc != 2){
	printf("Usage: command.dat\n");
	return  -1;
  }
 
  char* inputFile = argv[1];
  string filePath = inputFile;
  //  int pos = filePath.find_last_of("\\",filePath.size()-1);
  //  filePath.erase(pos,filePath.size()-1); 
 
  //load file
  TNR_INI INIF;
  bool result = false;
  
  printf("start load file\n");
  result = INIF.load(inputFile);

  if(result == false){
	TNR_ERR_write_message("file read err\n");
	return -1;
  }
  printf("end load file\n");

  //INIF.save("ookubo.dat");
  //Operation

  vector<TNR_SECTION*> operation;
  INIF.getSection("operation",operation);

  if(operation.size() == 0){
    TNR_ERR_write_message("operation section not found\n");
	return -1;
  }

  //section id numbering
  for(int i = 0;i < INIF.getNumSection();i++){
	INIF.getSection(i)->addKeyIntValue("section_id",i);
  }

  TNR_SECTION* ope = operation[0];
 
  TNR_MESH_MODEL model;

  for(int i = 0;i < ope->getNumData();i++){
    //get one command line.
    string com = ope->getData(i);
	//parse command line.
    TNR_COMAMND commands(com);

	//create mesh
	if(commands.getCommandName() == "MESH_CREATE"){
	  string input_name =  commands.getStringValue("input_name","");
	  string mesh_name =  commands.getStringValue("mesh_name","");
 
	  // shape comannd  expand to face or body.
	  vector<TNR_SECTION*> shapeSection;
	  INIF.getSection("shape","name",input_name,shapeSection);

	  for(int j = 0; j < shapeSection.size(); j++){

	    if(shapeSection[j]->getStringValue("geometry_type","") == "rectangle"){
          shape_make_rectangle(shapeSection[j],&commands);
	    }
	    if(shapeSection[j]->getStringValue("geometry_type","") == "circle"){
          shape_make_circle(shapeSection[j],&commands);
	    } 
	    if(shapeSection[j]->getStringValue("geometry_type","") == "oval"){
          shape_make_oval(shapeSection[j],&commands);
	    }  
	  }


	  //create mesh only face or body.
	  vector<TNR_SECTION*> sections;
	  INIF.getSection("name",input_name,sections);
	  
	  if(sections.size() == 0){ 
        TNR_ERR_write_message("name do not found or duplicate");
 	    return -1;  	
	  }


	  for(int j = 0; j < sections.size(); j++){

	    if(sections[j]->getName() == "face"){

	      //create mesh
	      TNR_QUAD_MESH* m = new TNR_QUAD_MESH(mesh_name);
		  m->createMesh(sections[j],&commands);
	      model.addMeshBody(m);
	    }

 	    if(sections[j]->getName() == "body"){

	      //create mesh
	      TNR_HEX_MESH* m = new TNR_HEX_MESH(mesh_name);
	      TNR_hex_create_mesh(sections[j],&commands,m);
	      model.addMeshBody(m);
	    }//end of one section

	

	  }//end of all section
	  //delete temp section
	  //string saveTempFileName = filePath + "\\" + "debug.ini";
      //INIF.save(saveTempFileName.c_str());
	  //INIF.deleteSectionByKey("section_id","");
	}//end of MESH_CREATE

	//mesh print
	if(commands.getCommandName() == "MESH_PRINT"){
	  string input_name  = commands.getStringValue("input_mesh_name","");
	  string output_name = commands.getStringValue("output_name","default");
	  FILE* fp = fopen(output_name.c_str(),"w");
	  TNR_vtu_header_write(fp);

	  // search out put mesh model
      vector<TNR_MESH*> searchedModel;
	  model.searchName(input_name,searchedModel);

	  //print out put model
	  for(int i = 0; i < model.getModelSize();i++){
		TNR_MESH* m = model[i];
	    if(m->getType() == "QUAD"){
          TNR_QUAD_MESH* qm= (TNR_QUAD_MESH*)m;
		  TNR_vtu_surfaceMesh_write(fp,qm);
	    }
	    if(m->getType() == "HEX"){
          TNR_HEX_MESH* qm= (TNR_HEX_MESH*)m;
		  TNR_vtu_solidMesh_write(fp,qm);
	    }
	  }

	  TNR_vtu_footer_write(fp);
      fclose(fp);
	}


	if(commands.getCommandName() == "MODEL_COPY"){
	  string input_name  =  commands.getStringValue("input_name","");
	  string output_name =  commands.getStringValue("output_name","default");

	  vector<TNR_SECTION*> sections;
	  INIF.getSection("name",input_name,sections);
	  if(sections.size() != 1){ 
        TNR_ERR_write_message("name do not found or duplicate");
 	    return -1;  	
	  }
      TNR_SECTION* org = sections[0];
      TNR_SECTION* s = new TNR_SECTION;
	  org->copy(*s);
	  s->addKeyStringValue("name",output_name);
	  INIF.addSection(s);
	  
 	}

	if(commands.getCommandName() == "MODEL_TRANSFORM"){
	  TNR_POINT tran;
	  string input_name  =  commands.getStringValue("input_name","");
	  string type =  commands.getStringValue("type","");
	  string center =  commands.getStringValue("center","0.0,0.0,0.0");
	  tran[0]  =  commands.getDoubleValue("x",0.0);
	  tran[1]  =  commands.getDoubleValue("y",0.0);
	  tran[2]  =  commands.getDoubleValue("z",0.0);

	  vector<TNR_SECTION*> sections;
	  INIF.getSection("name",input_name,sections);
	  if(sections.size() != 1){ 
        TNR_ERR_write_message("name do not found or duplicate");
 	    return -1;  	
	  }
      TNR_SECTION* s = sections[0];

      TNR_section_transration(s,type,center,TNR_POINT (0,0,0),tran);
  
 	}

  }

 


  return 0;
}